/*
 *
 *              FX-890P/Z-1 Binary file transfer support utility
 *
 *                     Copyright (C) 1993-97 by Y.Akagawa
 *
 * $Header: $
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#ifdef WIN32
  #include "comm.h"
#else
  #include "rs232c.h"
#endif

#define M_SEND	1
#define M_REC	2
#define M_TEXT  0
#define M_BIN   1

#define BUFFER_SIZE	16384

int send( FILE *fp );
int rec( FILE *fp );
int check_header( FILE *fp, unsigned long fs );
void check_cmdlin( int argc, char **argv );
int get( FILE *fp );
void prt_usage( void );

#ifdef PC98
const char *msg_title     = "File Transfer Utility Z1BIN Version 1.05/PC98";
const char *msg_request   = "%sbpsőMĂ";
const char *msg_init_fail = "̋@ł19200bpsȏŒʐM͏o܂";
#else
 #ifdef WIN32
	const char *msg_title     = "File Transfer Utility Z1BIN Version 1.05/Win32";
 #else
	const char *msg_title     = "File Transfer Utility Z1BIN Version 1.05/MCD";
 #endif
const char *msg_request   = "%dbpsőMĂ";
const char *msg_init_fail = "Ɏs܂";
#endif

int flg_mode,
	flg_txbn,
	flg_adr,
	flg_hdr,
	bPort,
	flow;

unsigned int start_addr,
			 run_addr,
			 bps;

char	*b[] = { "75", "150", "300", "600", "1200", "2400", "4800", 
				"9600", "19200", "38400" };

char buffer[BUFFER_SIZE];
unsigned int pointer = 0;
extern unsigned char _stop;

#ifdef WIN32
CComm comm;

int getaux()
{
	int	c = comm.ReadCommByte();
	if ( c < 0 )
		return( -2 );
	return c;
}

int putaux(int c)
{
	if(comm.WriteCommByte(c) == FALSE)
		return -2;
	return 0;
}
#endif


void main( int argc, char **argv )
{
	FILE *fp;
	int	res;
	char fmode[3];

	printf("%s\n", msg_title );
	printf("Copyright (c) 1992-97 by Y.Akagawa\n\n" );

	check_cmdlin( argc, argv );

	if( flg_mode == M_SEND )
	{
		strcpy( fmode, "rb" );
	}
	else
	{
		if( flg_mode == M_REC )
		{
			strcpy( fmode, "wb" );
		}
		else
		{
			prt_usage();
			exit( 1 );
		}
	}

#ifdef WIN32
	if(comm.OpenConnection((BYTE)bPort, (DWORD)bps, 8, ONESTOPBIT, NOPARITY, flow) == FALSE)
	{
		printf("OpenConnection failed.\n");
		exit(1);
	}
#else
	if( rs_init(bps, PARITY_NONE, STOP_1, CHR_8, flow) != 0)
	{
		printf("%s\n", msg_init_fail);
		exit( 1 );
	}
#endif
	if( ( fp = fopen( argv[1], fmode ) ) == NULL )
	{
		printf("%s I[vł܂\n", argv[1]);
		exit( 1 );
	}

	if( flg_mode == M_SEND )
		res = send( fp );
	else
		res = rec( fp );

	if( res == -1 )
		printf("\rf              \n");
	fclose( fp );
#ifdef WIN32
	comm.CloseConnection();
#endif
}


/*********************/
/* File Send Routine */
/* Entry:			 */
/*  FILE *fp;		 */
/* Return:			 */
/* -1 : abort		 */
/*  0 : normal		 */
/*********************/

int send( FILE *fp )
{
	int dat, i;
	unsigned long fs, step, mod, counter;
	unsigned char header[17];

	fs = (unsigned long)filelength( fileno( fp ) );	/* Get file size */

	if( flg_txbn == M_BIN )
	{
		i = check_header( fp, fs );
		fseek( fp, 0, SEEK_SET );
		if( i == -1 )							/* Header found ? */
		{
			header[0x00] = 0xff;				/* Header create */
			header[0x01] = 0xfe;
			header[0x02] = 0x4d;
			fs += 16;
			header[0x03] = (unsigned char)(fs & 0xff);
			header[0x04] = (unsigned char)((fs >> 8) & 0xff);
			header[0x05] = (unsigned char)((fs >> 16) & 0xff);
			header[0x06] = 0;
			header[0x07] = 0;
			header[0x08] = (unsigned char)(start_addr & 0xff);
			header[0x09] = (unsigned char)((start_addr >> 8) & 0xff);
			header[0x0a] = 0;
			header[0x0b] = 0;
			header[0x0c] = (unsigned char)(run_addr & 0xff);
			header[0x0d] = (unsigned char)((run_addr >> 8) & 0xff);
			header[0x0e] = 0;
			header[0x0f] = 0;
		}
		else
		{
			for(i=0; i<16; i++)
				header[i] = (unsigned char)fgetc(fp);
			fs = (unsigned long)header[0x03];
			fs += (unsigned long)header[0x04] * 0x100;
			fs += (unsigned long)header[0x05] * 0x10000;
			if( flg_adr )
			{
				header[0x03] = (unsigned char)(fs & 0xff);
				header[0x04] = (unsigned char)((fs >> 8) & 0xff);
				header[0x05] = (unsigned char)((fs >> 16) & 0xff);
			}
		}

		printf("load addr:%02X%02X run addr:%02X%02X size:%06lX\n",
		 header[9],header[8],header[13],header[12], fs-16);

		if( flg_hdr )
		{
			for( i=0; i<16; i++ )					/* Send header */
			{
				printf("W\r");
				if( putaux( header[i] ) == -2 )
					return( -1 );
				printf("S\r");
				fs--;
			}
		}
		
		printf("\r|----------|\r|");
		step = fs / 10;
		mod = fs % 10;
		counter = 0;

		while( fs-- )
		{
			dat = fgetc( fp );
			if( dat == EOF )
				break;
			if( putaux( dat ) == -2 )				/* Break ? */
				return( -1 );
			counter++;
			if(counter == step)
			{
				counter = 0;
				printf("o");
			}
		}
	}
	else
	{
		printf("Text mode. size:%06lX\n", fs);
		while( ( dat = fgetc( fp ) ) != EOF )
		{
			if( putaux( dat ) == -2 )				/* Break ? */
				return( -1 );
			if( dat == 0x1a )
				break;
		}
		if( dat != 0x1a )
			putaux( 0x1a );
	}

	printf("\a\r|M|");

	return( 0 );
}


/************************/
/* File Receive Routine */
/* Entry:			    */
/*  FILE *fp;			*/
/* Return:			    */
/* -1 : abort			*/
/*  0 : normal			*/
/************************/

int rec( FILE *fp )
{
	unsigned int  laddr, raddr;
	unsigned long step, mod, size, counter;
	int dat, res;

#ifdef PC98
	printf(msg_request, b[bps]);
#else
	printf(msg_request, bps);
#endif

#ifdef WIN32
	if( comm.WaitComm() == FALSE )
		return( -1 );
#else
	while((res = checkaux()) == -1 );
	if(res == -2)
		return(-1);
#endif
	printf("\r");
	if( flg_txbn == M_BIN )
	{
		if( get( fp ) == -1 )
			return( -1 );
		if( get( fp ) == -1 )
			return( -1 );
		if( get( fp ) == -1 )
			return( -1 );

		size = (unsigned long)get(fp);
		size += (unsigned long)get(fp)*0x100;
		size += (unsigned long)get(fp)*0x10000;
		size -= 0x10;

		if( get( fp ) == -1 )
			return( -1 );
		if( get( fp ) == -1 )
			return( -1 );

		laddr = (unsigned int)get( fp );
		laddr += (unsigned int)get( fp ) * 0x100;

		if( get( fp ) == -1 )
			return( -1 );
		if( get( fp ) == -1 )
			return( -1 );

		raddr = (unsigned int)get( fp );
		raddr += (unsigned int)get( fp ) * 0x100;

		if( get( fp ) == -1 )
			return( -1 );
		if( get( fp ) == -1 )
			return( -1 );

		if( flg_hdr == 0 )
			pointer = 0;

		printf("load addr:%04X run addr:%04X size:%06lX\n",
		 laddr, raddr, size);

		printf("|----------|\r|");
		step = size / 10;
		mod = size % 10;
		counter = 0;
		while( size-- )
		{
			dat = get( fp );
			if( dat == -1 )						/* break ? */
			{
				return( -1 );
			}
			counter++;
			if(counter == step)
			{
				counter = 0;
				printf("o");
			}
		}
		if(pointer)
			fwrite(buffer, pointer, 1, fp);
	}
	else
	{
		printf("\rText mode.                \n");
		get( fp );
		printf("\r|M|");

		do
		{
			dat = get( fp );
			if( dat == -1 )						/* break ? */
				return( -1 );
		}while( dat != 0x1a );
	}
	printf("\a\r|M|");

	return( 0 );
}


/******************************/
/* Command line check Routine */
/* Entry:					  */
/*       int argc;			  */
/*       char **argv;		  */
/******************************/

void check_cmdlin( int argc, char **argv )
{
	int  i;
	unsigned int b;
	char *stpstr;

	if( argc < 2 )
	{
		prt_usage();
		exit( -1 );
	}

	bps         =  BPS_9600;		/* ]x */
	start_addr  =  0x2000;			/* 擪AhX */
	run_addr    =  0;				/* sAhX */
	flg_mode    =  M_SEND;			/* ][h */
	flg_txbn    =  M_BIN;			/* oCi/eLXgtO */
	flg_adr     =  0;				/* 擪AhXwtO */
	flg_hdr		=  1;				/* wb_ttO */
	bPort       =  1;				/* Com|[gԍ */
	flow		=  FC_NONE;			/* t[ */
	if( argc > 2 )
	{
		for( i = 2; i < argc; i++ )
		{
			if(toupper(argv[i][1]) == 'M')
			{
				if(toupper(argv[i][2]) == 'S')
					flg_mode = M_SEND;
				else if(toupper(argv[i][2]) == 'R')
					flg_mode = M_REC;
				else
				{
					prt_usage();
					exit(-1);
				}
			}
			if( toupper( argv[i][1] ) == 'S' )
			{
				flg_adr = 1;
				start_addr = (unsigned int)strtoul( argv[i]+2, &stpstr, 16 );
			}
			if( toupper( argv[i][1] ) == 'P' )
			{
				bPort = atoi(argv[i] + 2);
			}

#ifdef PC98
			if(toupper(argv[i][1]) == 'X')
			{
				flow = XFLOW_ON;
			}
#else
			if( toupper( argv[i][1] ) == 'F' )
			{
				switch( atoi(argv[i] + 2) )
				{
					case 0 :
						flow = FC_NONE;
						break;
					case 1 :
						flow = FC_RTSCTS;
						break;
					case 2 :
						flow = FC_SXONXOFF;
						break;
					default : 
						flow = FC_NONE;
						break;
				}
			}

#endif
			if( toupper( argv[i][1] ) == 'T' )
			{
				flg_txbn = M_TEXT;
			}

			if( toupper( argv[i][1] ) == 'N' )
			{
				flg_hdr = 0;
			}
			if( toupper( argv[i][1] ) == 'B' )
			{
				bps = (unsigned int) atoi( argv[i]+2 );
				b = bps / 100;
				if( bps != b * 100 )
					bps = 0;
				else
					bps = b;
				switch( bps )
				{
					case 3 :
						bps = BPS_300;
						break;
					case 6 :
						bps = BPS_600;
						break;
					case 12 :
						bps = BPS_1200;
						break;
					case 24 :
						bps = BPS_2400;
						break;
					case 48 :
						bps = BPS_4800;
						break;
					case 96 :
						bps = BPS_9600;
						break;
					case 192 :
						bps = BPS_19200;
						break;
					case 384 :
						bps = BPS_38400;
						break;
					default :
						prt_usage();
						exit( -1 );
				}
			}
		}
		if( flg_mode == M_REC )
		{
			if( flow == FC_SXONXOFF )
				flow = FC_RXONXOFF;
		}
	}
}


/************************/
/* Header check Routine */
/* Entry:				*/
/*  FILE *fp;			*/
/*  long filesize;		*/
/* Return:				*/
/* -1 : no Header		*/
/*  0 : Header found	*/
/************************/

int check_header( FILE *fp, unsigned long fs )
{
	if( fs < 16 )
		return( -1 );
	if( fgetc( fp ) != 0xff )
		return( -1 );
	if( fgetc( fp ) != 0xfe )
		return( -1 );
	if( fgetc( fp ) != 0x4d )
		return( -1 );
	return( 0 );
}


/**************************/
/* Receive AUX & Put file */
/* Entry:				  */
/*  FILE *fp;			  */
/* Return:				  */
/* -1 : Break			  */
/*  * : Receive data	  */
/**************************/

int get( FILE *fp )
{
	int	dat;
	do
	{
		dat = getaux();
		if( dat == -2 )
			return( -1 );
	}while( dat == -1 );
	buffer[pointer++] = (char)dat;
	if(pointer == BUFFER_SIZE)
	{
		fwrite(buffer, BUFFER_SIZE, 1, fp);
		pointer = 0;
	}
	return( dat );
}

/**************************/
/* Display usage          */
/**************************/

void prt_usage( void )
{
	printf("Usage : Z1BIN filename [option]\n\n");
	printf("option : -Sxxxx    i[JnAhXiPUij\n");
	printf("         -MR       t@CM\n");
	printf("         -MS       t@CM\n");
	printf("         -Bxxxx    ]xi300`38400j\n");
	printf("         -N        wb_tȂ\n");
	printf("         -T        eLXg[h\n");
#ifdef WIN32	
	printf("         -Px       |[g\n");
#endif
#ifdef PC98
	printf("         -X        w䂠(M̂)\n");
#else
	printf("         -Fx       t[(0:Ȃ,1:n[h,2:\tg)\n");
#endif
}



